@@ -134,6 +134,9 @@ enum lttng_error_code {
 	LTTNG_ERR_EXCLUSION_NOMEM        = 111, /* Lack of memory while processing event exclusions */
 	LTTNG_ERR_INVALID_EVENT_NAME     = 112, /* Invalid event name */
 	LTTNG_ERR_INVALID_CHANNEL_NAME   = 113, /* Invalid channel name */
+	LTTNG_ERR_PID_TRACKED            = 114, /* PID already tracked */
+	LTTNG_ERR_PID_NOT_TRACKED        = 115, /* PID not tracked */
+	LTTNG_ERR_INVALID_CHANNEL_DOMAIN = 116, /* Invalid channel domain */
 
 	/* MUST be last element */
 	LTTNG_ERR_NR,                           /* Last element */
