@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2013 - Julien Desfossez <jdesfossez@efficios.com>
  *                      David Goulet <dgoulet@efficios.com>
+ *               2015 - Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License, version 2 only, as
@@ -17,6 +18,7 @@
  */
 
 #define _GNU_SOURCE
+#define _LGPL_SOURCE
 #include <assert.h>
 #include <string.h>
 
@@ -26,40 +28,28 @@
 #include <common/compat/endian.h>
 
 #include "cmd-generic.h"
-#include "cmd-2-1.h"
-#include "utils.h"
+#include "lttng-relayd.h"
 
-int cmd_recv_stream_2_2(struct relay_connection *conn,
-		struct relay_stream *stream)
+int cmd_create_session_2_4(struct relay_connection *conn,
+		char *session_name, char *hostname,
+		uint32_t *live_timer, bool *snapshot)
 {
 	int ret;
-	struct lttcomm_relayd_add_stream_2_2 stream_info;
+	struct lttcomm_relayd_create_session_2_4 session_info;
 
-	assert(conn);
-	assert(stream);
-
-	ret = cmd_recv(conn->sock, &stream_info, sizeof(stream_info));
+	ret = cmd_recv(conn->sock, &session_info, sizeof(session_info));
 	if (ret < 0) {
-		ERR("Unable to recv stream version 2.2");
+		ERR("Unable to recv session info version 2.4");
 		goto error;
 	}
 
-	stream->path_name = create_output_path(stream_info.pathname);
-	if (stream->path_name == NULL) {
-		PERROR("Path name allocation");
-		ret = -ENOMEM;
-		goto error;
-	}
+	strncpy(session_name, session_info.session_name,
+			sizeof(session_info.session_name));
+	strncpy(hostname, session_info.hostname,
+			sizeof(session_info.hostname));
+	*live_timer = be32toh(session_info.live_timer);
+	*snapshot = be32toh(session_info.snapshot);
 
-	stream->channel_name = strdup(stream_info.channel_name);
-	if (stream->channel_name == NULL) {
-		ret = -errno;
-		PERROR("Path name allocation");
-		goto error;
-	}
-
-	stream->tracefile_size = be64toh(stream_info.tracefile_size);
-	stream->tracefile_count = be64toh(stream_info.tracefile_count);
 	ret = 0;
 
 error:
