@@ -16,6 +16,7 @@
  */
 
 #define _GNU_SOURCE
+#define _LGPL_SOURCE
 #include <assert.h>
 
 #include <common/common.h>
@@ -27,6 +28,7 @@
 
 #include "fd-limit.h"
 #include "agent-thread.h"
+#include "agent.h"
 #include "lttng-sessiond.h"
 #include "session.h"
 #include "utils.h"
@@ -72,53 +74,6 @@ static void update_agent_app(struct agent_app *app)
 	session_unlock_list();
 }
 
-/*
- * Destroy a agent application by socket.
- */
-static void destroy_agent_app(int sock)
-{
-	struct agent_app *app;
-
-	assert(sock >= 0);
-
-	/*
-	 * Not finding an application is a very important error that should NEVER
-	 * happen. The hash table deletion is ONLY done through this call even on
-	 * thread cleanup.
-	 */
-	rcu_read_lock();
-	app = agent_find_app_by_sock(sock);
-	assert(app);
-	rcu_read_unlock();
-
-	/* RCU read side lock is taken in this function call. */
-	agent_delete_app(app);
-
-	/* The application is freed in a RCU call but the socket is closed here. */
-	agent_destroy_app(app);
-}
-
-/*
- * Cleanup remaining agent apps in the hash table. This should only be called in
- * the exit path of the thread.
- */
-static void clean_agent_apps_ht(void)
-{
-	struct lttng_ht_node_ulong *node;
-	struct lttng_ht_iter iter;
-
-	DBG3("[agent-thread] Cleaning agent apps ht");
-
-	rcu_read_lock();
-	cds_lfht_for_each_entry(agent_apps_ht_by_sock->ht, &iter.iter, node, node) {
-		struct agent_app *app;
-
-		app = caa_container_of(node, struct agent_app, node);
-		destroy_agent_app(app->sock->fd);
-	}
-	rcu_read_unlock();
-}
-
 /*
  * Create and init socket from uri.
  */
@@ -341,39 +296,26 @@ restart:
 				goto exit;
 			}
 
-			/*
-			 * Check first if this is a POLLERR since POLLIN is also included
-			 * in an error value thus checking first.
-			 */
-			if (revents & (LPOLLERR | LPOLLHUP | LPOLLRDHUP)) {
-				/* Removing from the poll set */
-				ret = lttng_poll_del(&events, pollfd);
-				if (ret < 0) {
-					goto error;
-				}
-
-				destroy_agent_app(pollfd);
-			} else if (revents & (LPOLLIN)) {
+			if (revents & LPOLLIN) {
 				int new_fd;
 				struct agent_app *app = NULL;
 
-				/* Pollin event of agent app socket should NEVER happen. */
 				assert(pollfd == reg_sock->fd);
-
 				new_fd = handle_registration(reg_sock, &app);
 				if (new_fd < 0) {
-					WARN("[agent-thread] agent registration failed. Ignoring.");
-					/* Somehow the communication failed. Just continue. */
 					continue;
 				}
 				/* Should not have a NULL app on success. */
 				assert(app);
 
-				/* Only add poll error event to only detect shutdown. */
+				/*
+				 * Since this is a command socket (write then read),
+				 * only add poll error event to only detect shutdown.
+				 */
 				ret = lttng_poll_add(&events, new_fd,
 						LPOLLERR | LPOLLHUP | LPOLLRDHUP);
 				if (ret < 0) {
-					destroy_agent_app(new_fd);
+					agent_destroy_app_by_sock(new_fd);
 					continue;
 				}
 
@@ -381,10 +323,26 @@ restart:
 				update_agent_app(app);
 
 				/* On failure, the poll will detect it and clean it up. */
-				(void) agent_send_registration_done(app);
+				ret = agent_send_registration_done(app);
+				if (ret < 0) {
+					/* Removing from the poll set */
+					ret = lttng_poll_del(&events, new_fd);
+					if (ret < 0) {
+						goto error;
+					}
+					agent_destroy_app_by_sock(new_fd);
+					continue;
+				}
+			} else if (revents & (LPOLLERR | LPOLLHUP | LPOLLRDHUP)) {
+				/* Removing from the poll set */
+				ret = lttng_poll_del(&events, pollfd);
+				if (ret < 0) {
+					goto error;
+				}
+				agent_destroy_app_by_sock(pollfd);
 			} else {
-				ERR("Unknown poll events %u for sock %d", revents, pollfd);
-				continue;
+				ERR("Unexpected poll events %u for sock %d", revents, pollfd);
+				goto error;
 			}
 		}
 	}
@@ -399,11 +357,6 @@ error_tcp_socket:
 error_poll_create:
 	DBG("[agent-thread] is cleaning up and stopping.");
 
-	if (agent_apps_ht_by_sock) {
-		clean_agent_apps_ht();
-		lttng_ht_destroy(agent_apps_ht_by_sock);
-	}
-
 	rcu_thread_offline();
 	rcu_unregister_thread();
 	return NULL;
