@@ -28,8 +28,7 @@
 #define KERN_MODULES_MINOR         0
 
 struct kern_modules_param {
-	const char *name;
-	int required;
+	char *name;
 };
 
 #endif /* _KERN_MODULES_H */
