@@ -58,5 +58,7 @@ extern int cmd_disable_consumer(int argc, const char **argv);
 extern int cmd_snapshot(int argc, const char **argv);
 extern int cmd_save(int argc, const char **argv);
 extern int cmd_load(int argc, const char **argv);
+extern int cmd_track(int argc, const char **argv);
+extern int cmd_untrack(int argc, const char **argv);
 
 #endif /* _LTTNG_CMD_H */
