@@ -16,6 +16,7 @@
  */
 
 #define _GNU_SOURCE
+#define _LGPL_SOURCE
 #include <ctype.h>
 #include <popt.h>
 #include <stdio.h>
@@ -24,6 +25,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <assert.h>
 
 #include <urcu/list.h>
 
@@ -567,9 +569,7 @@ static int add_context(char *session_name)
 	} else if (opt_userspace) {
 		dom.type = LTTNG_DOMAIN_UST;
 	} else {
-		print_missing_domain();
-		ret = CMD_ERROR;
-		goto error;
+		assert(0);
 	}
 
 	handle = lttng_create_handle(session_name, &dom);
@@ -717,7 +717,7 @@ int cmd_add_context(int argc, const char **argv)
 
 			type = zmalloc(sizeof(struct ctx_type));
 			if (type == NULL) {
-				perror("malloc ctx_type");
+				PERROR("malloc ctx_type");
 				ret = CMD_FATAL;
 				goto end;
 			}
@@ -748,6 +748,13 @@ int cmd_add_context(int argc, const char **argv)
 		}
 	}
 
+	ret = print_missing_or_multiple_domains(opt_kernel + opt_userspace);
+
+	if (ret) {
+		ret = CMD_ERROR;
+		goto end;
+	}
+
 	if (!opt_type) {
 		ERR("Missing mandatory -t TYPE");
 		usage(stderr);
