@@ -168,14 +168,23 @@
 /* Default JUL tracepoint name. This is a wildcard for the JUL domain. */
 #define DEFAULT_SYS_JUL_EVENT_NAME      "lttng_jul:sys*"
 #define DEFAULT_USER_JUL_EVENT_NAME     "lttng_jul:user*"
+#define DEFAULT_JUL_EVENT_COMPONENT     "lttng_jul"
+#define DEFAULT_JUL_EVENT_NAME          DEFAULT_JUL_EVENT_COMPONENT ":*"
 
 /* Default log4j domain channel name. */
 #define DEFAULT_LOG4J_CHANNEL_NAME        "lttng_log4j_channel"
 /* Default log4j tracepoint name. This is a wildcard for the log4j domain. */
 #define DEFAULT_SYS_LOG4J_EVENT_NAME      "lttng_log4j:sys*"
 #define DEFAULT_USER_LOG4J_EVENT_NAME     "lttng_log4j:user*"
+#define DEFAULT_LOG4J_EVENT_COMPONENT     "lttng_log4j"
+#define DEFAULT_LOG4J_EVENT_NAME          DEFAULT_LOG4J_EVENT_COMPONENT ":*"
+
+/* Default Python domain channel name. */
+#define DEFAULT_PYTHON_CHANNEL_NAME       "lttng_python_channel"
+/* Default Python tracepoint name. This is a wildcard for the python domain. */
+#define DEFAULT_PYTHON_EVENT_COMPONENT    "lttng_python"
+#define DEFAULT_PYTHON_EVENT_NAME         DEFAULT_PYTHON_EVENT_COMPONENT ":*"
 
-/* JUL default channel name. */
 #define DEFAULT_CHANNEL_OVERWRITE       0
 #define DEFAULT_CHANNEL_TRACEFILE_SIZE  0
 #define DEFAULT_CHANNEL_TRACEFILE_COUNT 0
@@ -297,6 +306,9 @@
 /* Default lttng command live timer value in usec. */
 #define DEFAULT_LTTNG_LIVE_TIMER			1000000
 
+/* Default runas worker name */
+#define DEFAULT_RUN_AS_WORKER_NAME			"lttng-runas"
+
 extern size_t default_channel_subbuf_size;
 extern size_t default_metadata_subbuf_size;
 extern size_t default_ust_pid_channel_subbuf_size;
