@@ -119,6 +119,11 @@ struct lttng_kernel_tracer_version {
 	uint32_t patchlevel;
 } LTTNG_PACKED;
 
+struct lttng_kernel_tracer_abi_version {
+	uint32_t major;
+	uint32_t minor;
+} LTTNG_PACKED;
+
 enum lttng_kernel_calibrate_type {
 	LTTNG_KERNEL_CALIBRATE_KRETPROBE,
 };
@@ -127,6 +132,11 @@ struct lttng_kernel_calibrate {
 	enum lttng_kernel_calibrate_type type;	/* type (input) */
 } LTTNG_PACKED;
 
+struct lttng_kernel_syscall_mask {
+	uint32_t len;	/* in bits */
+	char mask[];
+} LTTNG_PACKED;
+
 /*
  * kernel channel
  */
@@ -142,4 +152,12 @@ struct lttng_kernel_channel {
 	char padding[LTTNG_KERNEL_CHANNEL_PADDING1];
 } LTTNG_PACKED;
 
+#define KERNEL_FILTER_BYTECODE_MAX_LEN		65536
+struct lttng_kernel_filter_bytecode {
+	uint32_t len;
+	uint32_t reloc_offset;
+	uint64_t seqnum;
+	char data[0];
+} LTTNG_PACKED;
+
 #endif /* _LTTNG_KERNEL_H */
