@@ -31,6 +31,7 @@
 #include <sched.h>
 #include <sys/signal.h>
 #include <assert.h>
+#include <sys/prctl.h>
 
 #include <common/common.h>
 #include <common/utils.h>
@@ -322,12 +323,11 @@ int run_as_worker(struct run_as_worker *worker)
 	memset(worker->procname, 0, proc_orig_len);
 	strncpy(worker->procname, DEFAULT_RUN_AS_WORKER_NAME, proc_orig_len);
 
-	ret = pthread_setname_np(pthread_self(), DEFAULT_RUN_AS_WORKER_NAME);
+	ret = prctl(PR_SET_NAME, DEFAULT_RUN_AS_WORKER_NAME, 0, 0, 0);
 	if (ret) {
-		errno = ret;
-		ret = -1;
-		PERROR("pthread_setname_np");
-		return EXIT_FAILURE;
+		/* Don't fail as this is not essential. */
+		PERROR("prctl PR_SET_NAME");
+		ret = 0;
 	}
 
 	sendret.ret = 0;
@@ -528,9 +528,9 @@ int run_as_rmdir_recursive(const char *path, uid_t uid, gid_t gid)
 }
 
 static
-void reset_sighandler(void)
+int reset_sighandler(void)
 {
-	int sig;
+	int sig, ret = 0;
 
 	for (sig = SIGHUP; sig <= SIGUNUSED; sig++) {
 		/* Skip unblockable signals. */
@@ -539,8 +539,67 @@ void reset_sighandler(void)
 		}
 		if (signal(sig, SIG_DFL) == SIG_ERR) {
 			PERROR("reset signal %d", sig);
+			ret = -1;
+			goto end;
 		}
 	}
+end:
+	return ret;
+}
+
+static
+void worker_sighandler(int sig)
+{
+	const char *signame;
+
+	/*
+	 * The worker will its parent's signals since they are part of the same
+	 * process group. However, in the case of SIGINT and SIGTERM, we want
+	 * to give the worker a chance to teardown gracefully when its parent
+	 * closes the command socket.
+	 */
+	switch (sig) {
+	case SIGINT:
+		signame = "SIGINT";
+		break;
+	case SIGTERM:
+		signame = "SIGTERM";
+		break;
+	default:
+		signame = "Unknown";
+	}
+
+	DBG("run_as worker received signal %s", signame);
+}
+
+static
+int set_worker_sighandlers(void)
+{
+	int ret = 0;
+	sigset_t sigset;
+	struct sigaction sa;
+
+	if ((ret = sigemptyset(&sigset)) < 0) {
+		PERROR("sigemptyset");
+		goto end;
+	}
+
+	sa.sa_handler = worker_sighandler;
+	sa.sa_mask = sigset;
+	sa.sa_flags = 0;
+	if ((ret = sigaction(SIGINT, &sa, NULL)) < 0) {
+		PERROR("sigaction SIGINT");
+		goto end;
+	}
+
+	if ((ret = sigaction(SIGTERM, &sa, NULL)) < 0) {
+		PERROR("sigaction SIGTERM");
+		goto end;
+	}
+
+	DBG("run_as signal handler set for SIGTERM and SIGINT");
+end:
+	return ret;
 }
 
 LTTNG_HIDDEN
@@ -584,6 +643,8 @@ int run_as_create_worker(char *procname)
 
 		reset_sighandler();
 
+		set_worker_sighandlers();
+
 		/* The child has no use for this lock. */
 		pthread_mutex_unlock(&worker_lock);
 		/* Just close, no shutdown. */
