@@ -20,6 +20,7 @@
  */
 
 #define _GNU_SOURCE
+#define _LGPL_SOURCE
 #include <assert.h>
 #include <grp.h>
 #include <errno.h>
@@ -74,6 +75,7 @@ static int connected;
  */
 int lttng_opt_quiet;
 int lttng_opt_verbose;
+int lttng_opt_mi;
 
 /*
  * Copy string from src to dst and enforce null terminated byte.
@@ -104,6 +106,8 @@ void lttng_ctl_copy_lttng_domain(struct lttng_domain *dst,
 		case LTTNG_DOMAIN_KERNEL:
 		case LTTNG_DOMAIN_UST:
 		case LTTNG_DOMAIN_JUL:
+		case LTTNG_DOMAIN_LOG4J:
+		case LTTNG_DOMAIN_PYTHON:
 			memcpy(dst, src, sizeof(struct lttng_domain));
 			break;
 		default:
@@ -217,19 +221,19 @@ int lttng_check_tracing_group(void)
 	/* Get number of supplementary group IDs */
 	grp_list_size = getgroups(0, NULL);
 	if (grp_list_size < 0) {
-		perror("getgroups");
+		PERROR("getgroups");
 		goto end;
 	}
 
 	/* Alloc group list of the right size */
-	grp_list = malloc(grp_list_size * sizeof(gid_t));
+	grp_list = zmalloc(grp_list_size * sizeof(gid_t));
 	if (!grp_list) {
-		perror("malloc");
+		PERROR("malloc");
 		goto end;
 	}
 	grp_id = getgroups(grp_list_size, grp_list);
 	if (grp_id < 0) {
-		perror("getgroups");
+		PERROR("getgroups");
 		goto free_list;
 	}
 
@@ -271,7 +275,7 @@ static int try_connect_sessiond(const char *sock_path)
 
 	ret = lttcomm_close_unix_sock(ret);
 	if (ret < 0) {
-		perror("lttcomm_close_unix_sock");
+		PERROR("lttcomm_close_unix_sock");
 	}
 
 	return 0;
@@ -442,7 +446,11 @@ int lttng_ctl_ask_sessiond_varlen(struct lttcomm_session_msg *lsm,
 		goto end;
 	}
 
-	data = (void*) malloc(size);
+	data = zmalloc(size);
+	if (!data) {
+		ret = -ENOMEM;
+		goto end;
+	}
 
 	/* Get payload data */
 	ret = recv_data_sessiond(data, size);
@@ -482,7 +490,7 @@ struct lttng_handle *lttng_create_handle(const char *session_name,
 		goto end;
 	}
 
-	handle = malloc(sizeof(struct lttng_handle));
+	handle = zmalloc(sizeof(struct lttng_handle));
 	if (handle == NULL) {
 		PERROR("malloc handle");
 		goto end;
@@ -688,25 +696,25 @@ int lttng_enable_event_with_filter(struct lttng_handle *handle,
 }
 
 /*
- * Depending on the event, return a newly allocated JUL filter expression or
+ * Depending on the event, return a newly allocated agent filter expression or
  * NULL if not applicable.
  *
  * An event with NO loglevel and the name is * will return NULL.
  */
-static char *set_jul_filter(const char *filter, struct lttng_event *ev)
+static char *set_agent_filter(const char *filter, struct lttng_event *ev)
 {
 	int err;
-	char *jul_filter = NULL;
+	char *agent_filter = NULL;
 
 	assert(ev);
 
 	/* Don't add filter for the '*' event. */
 	if (ev->name[0] != '*') {
 		if (filter) {
-			err = asprintf(&jul_filter, "(%s) && (logger_name == \"%s\")", filter,
+			err = asprintf(&agent_filter, "(%s) && (logger_name == \"%s\")", filter,
 					ev->name);
 		} else {
-			err = asprintf(&jul_filter, "logger_name == \"%s\"", ev->name);
+			err = asprintf(&agent_filter, "logger_name == \"%s\"", ev->name);
 		}
 		if (err < 0) {
 			PERROR("asprintf");
@@ -724,18 +732,18 @@ static char *set_jul_filter(const char *filter, struct lttng_event *ev)
 			op = "==";
 		}
 
-		if (filter || jul_filter) {
+		if (filter || agent_filter) {
 			char *new_filter;
 
 			err = asprintf(&new_filter, "(%s) && (int_loglevel %s %d)",
-					jul_filter ? jul_filter : filter, op,
+					agent_filter ? agent_filter : filter, op,
 					ev->loglevel);
-			if (jul_filter) {
-				free(jul_filter);
+			if (agent_filter) {
+				free(agent_filter);
 			}
-			jul_filter = new_filter;
+			agent_filter = new_filter;
 		} else {
-			err = asprintf(&jul_filter, "int_loglevel %s %d", op,
+			err = asprintf(&agent_filter, "int_loglevel %s %d", op,
 					ev->loglevel);
 		}
 		if (err < 0) {
@@ -744,9 +752,9 @@ static char *set_jul_filter(const char *filter, struct lttng_event *ev)
 		}
 	}
 
-	return jul_filter;
+	return agent_filter;
 error:
-	free(jul_filter);
+	free(agent_filter);
 	return NULL;
 }
 
@@ -824,6 +832,12 @@ static int generate_filter(char *filter_expression,
 		ret = -LTTNG_ERR_FILTER_INVAL;
 		goto parse_error;
 	}
+	/* Validate strings used as literals in the expression */
+	ret = filter_visitor_ir_validate_string(ctx);
+	if (ret) {
+		ret = -LTTNG_ERR_FILTER_INVAL;
+		goto parse_error;
+	}
 	dbg_printf("done\n");
 
 	dbg_printf("Generating bytecode... ");
@@ -844,7 +858,7 @@ static int generate_filter(char *filter_expression,
 
 	/* No need to keep the memory stream. */
 	if (fclose(fmem) != 0) {
-		perror("fclose");
+		PERROR("fclose");
 	}
 
 	*ctxp = ctx;
@@ -855,7 +869,7 @@ parse_error:
 	filter_parser_ctx_free(ctx);
 filter_alloc_error:
 	if (fclose(fmem) != 0) {
-		perror("fclose");
+		PERROR("fclose");
 	}
 error:
 	return ret;
@@ -911,13 +925,14 @@ int lttng_enable_event_with_exclusions(struct lttng_handle *handle,
 				sizeof(lsm.u.enable.channel_name));
 	}
 
-	if (ev->name[0] != '\0') {
-		lsm.cmd_type = LTTNG_ENABLE_EVENT;
-	} else {
-		lsm.cmd_type = LTTNG_ENABLE_ALL_EVENT;
+	lsm.cmd_type = LTTNG_ENABLE_EVENT;
+	if (ev->name[0] == '\0') {
+		/* Enable all events */
+		lttng_ctl_copy_string(ev->name, "*", sizeof(ev->name));
 	}
 
 	lttng_ctl_copy_lttng_domain(&lsm.domain, &handle->domain);
+	/* FIXME: copying non-packed struct to packed struct. */
 	memcpy(&lsm.u.enable.event, ev, sizeof(lsm.u.enable.event));
 
 	lttng_ctl_copy_string(lsm.session.name, handle->session_name,
@@ -931,7 +946,9 @@ int lttng_enable_event_with_exclusions(struct lttng_handle *handle,
 	 * filtering by logger name.
 	 */
 	if (exclusion_count == 0 && filter_expression == NULL &&
-			handle->domain.type != LTTNG_DOMAIN_JUL) {
+			(handle->domain.type != LTTNG_DOMAIN_JUL &&
+				handle->domain.type != LTTNG_DOMAIN_LOG4J &&
+				handle->domain.type != LTTNG_DOMAIN_PYTHON)) {
 		goto ask_sessiond;
 	}
 
@@ -941,23 +958,27 @@ int lttng_enable_event_with_exclusions(struct lttng_handle *handle,
 	 */
 
 	/* Parse filter expression */
-	if (filter_expression != NULL || handle->domain.type == LTTNG_DOMAIN_JUL) {
-		if (handle->domain.type == LTTNG_DOMAIN_JUL) {
-			char *jul_filter;
+	if (filter_expression != NULL || handle->domain.type == LTTNG_DOMAIN_JUL
+			|| handle->domain.type == LTTNG_DOMAIN_LOG4J
+			|| handle->domain.type == LTTNG_DOMAIN_PYTHON) {
+		if (handle->domain.type == LTTNG_DOMAIN_JUL ||
+				handle->domain.type == LTTNG_DOMAIN_LOG4J ||
+				handle->domain.type == LTTNG_DOMAIN_PYTHON) {
+			char *agent_filter;
 
 			/* Setup JUL filter if needed. */
-			jul_filter = set_jul_filter(filter_expression, ev);
-			if (!jul_filter) {
+			agent_filter = set_agent_filter(filter_expression, ev);
+			if (!agent_filter) {
 				if (!filter_expression) {
 					/* No JUL and no filter, just skip everything below. */
 					goto ask_sessiond;
 				}
 			} else {
 				/*
-				 * With a JUL filter, the original filter has been added to it
-				 * thus replace the filter expression.
+				 * With an agent filter, the original filter has been added to
+				 * it thus replace the filter expression.
 				 */
-				filter_expression = jul_filter;
+				filter_expression = agent_filter;
 				free_filter_expression = 1;
 			}
 		}
@@ -1028,19 +1049,34 @@ ask_sessiond:
 	return ret;
 }
 
-/*
- *  Disable event(s) of a channel and domain.
- *  If no event name is specified, all events are disabled.
- *  If no channel name is specified, the default 'channel0' is used.
- *  Returns size of returned session payload data or a negative error code.
- */
-int lttng_disable_event(struct lttng_handle *handle, const char *name,
-		const char *channel_name)
+int lttng_disable_event_ext(struct lttng_handle *handle,
+		struct lttng_event *ev, const char *channel_name,
+		const char *original_filter_expression)
 {
 	struct lttcomm_session_msg lsm;
+	char *varlen_data;
+	int ret = 0;
+	unsigned int free_filter_expression = 0;
+	struct filter_parser_ctx *ctx = NULL;
+	/*
+	 * Cast as non-const since we may replace the filter expression
+	 * by a dynamically allocated string. Otherwise, the original
+	 * string is not modified.
+	 */
+	char *filter_expression = (char *) original_filter_expression;
 
-	if (handle == NULL) {
-		return -LTTNG_ERR_INVALID;
+	if (handle == NULL || ev == NULL) {
+		ret = -LTTNG_ERR_INVALID;
+		goto error;
+	}
+
+	/* Empty filter string will always be rejected by the parser
+	 * anyway, so treat this corner-case early to eliminate
+	 * lttng_fmemopen error for 0-byte allocation.
+	 */
+	if (filter_expression && filter_expression[0] == '\0') {
+		ret = -LTTNG_ERR_INVALID;
+		goto error;
 	}
 
 	memset(&lsm, 0, sizeof(lsm));
@@ -1054,20 +1090,132 @@ int lttng_disable_event(struct lttng_handle *handle, const char *name,
 				sizeof(lsm.u.disable.channel_name));
 	}
 
-	lttng_ctl_copy_lttng_domain(&lsm.domain, &handle->domain);
+	lsm.cmd_type = LTTNG_DISABLE_EVENT;
 
-	if (name != NULL && *name != '*') {
-		lttng_ctl_copy_string(lsm.u.disable.name, name,
-				sizeof(lsm.u.disable.name));
-		lsm.cmd_type = LTTNG_DISABLE_EVENT;
-	} else {
-		lsm.cmd_type = LTTNG_DISABLE_ALL_EVENT;
-	}
+	lttng_ctl_copy_lttng_domain(&lsm.domain, &handle->domain);
+	/* FIXME: copying non-packed struct to packed struct. */
+	memcpy(&lsm.u.disable.event, ev, sizeof(lsm.u.disable.event));
 
 	lttng_ctl_copy_string(lsm.session.name, handle->session_name,
 			sizeof(lsm.session.name));
+	lsm.u.disable.bytecode_len = 0;
 
-	return lttng_ctl_ask_sessiond(&lsm, NULL);
+	/*
+	 * For the JUL domain, a filter is enforced except for the
+	 * disable all event. This is done to avoid having the event in
+	 * all sessions thus filtering by logger name.
+	 */
+	if (filter_expression == NULL &&
+			(handle->domain.type != LTTNG_DOMAIN_JUL &&
+				handle->domain.type != LTTNG_DOMAIN_LOG4J &&
+				handle->domain.type != LTTNG_DOMAIN_PYTHON)) {
+		goto ask_sessiond;
+	}
+
+	/*
+	 * We have a filter, so we need to set up a variable-length
+	 * memory block from where to send the data.
+	 */
+
+	/* Parse filter expression */
+	if (filter_expression != NULL || handle->domain.type == LTTNG_DOMAIN_JUL
+			|| handle->domain.type == LTTNG_DOMAIN_LOG4J
+			|| handle->domain.type == LTTNG_DOMAIN_PYTHON) {
+		if (handle->domain.type == LTTNG_DOMAIN_JUL ||
+				handle->domain.type == LTTNG_DOMAIN_LOG4J ||
+				handle->domain.type == LTTNG_DOMAIN_PYTHON) {
+			char *agent_filter;
+
+			/* Setup JUL filter if needed. */
+			agent_filter = set_agent_filter(filter_expression, ev);
+			if (!agent_filter) {
+				if (!filter_expression) {
+					/* No JUL and no filter, just skip everything below. */
+					goto ask_sessiond;
+				}
+			} else {
+				/*
+				 * With a JUL filter, the original filter has been added to it
+				 * thus replace the filter expression.
+				 */
+				filter_expression = agent_filter;
+				free_filter_expression = 1;
+			}
+		}
+
+		ret = generate_filter(filter_expression, &lsm, &ctx);
+		if (ret) {
+			goto filter_error;
+		}
+	}
+
+	varlen_data = zmalloc(lsm.u.disable.bytecode_len
+			+ lsm.u.disable.expression_len);
+	if (!varlen_data) {
+		ret = -LTTNG_ERR_EXCLUSION_NOMEM;
+		goto mem_error;
+	}
+
+	/* Add filter expression */
+	if (lsm.u.disable.expression_len != 0) {
+		memcpy(varlen_data,
+			filter_expression,
+			lsm.u.disable.expression_len);
+	}
+	/* Add filter bytecode next */
+	if (ctx && lsm.u.disable.bytecode_len != 0) {
+		memcpy(varlen_data
+			+ lsm.u.disable.expression_len,
+			&ctx->bytecode->b,
+			lsm.u.disable.bytecode_len);
+	}
+
+	ret = lttng_ctl_ask_sessiond_varlen(&lsm, varlen_data,
+			lsm.u.disable.bytecode_len + lsm.u.disable.expression_len, NULL);
+	free(varlen_data);
+
+mem_error:
+	if (filter_expression && ctx) {
+		filter_bytecode_free(ctx);
+		filter_ir_free(ctx);
+		filter_parser_ctx_free(ctx);
+	}
+filter_error:
+	if (free_filter_expression) {
+		/*
+		 * The filter expression has been replaced and must be freed as it is
+		 * not the original filter expression received as a parameter.
+		 */
+		free(filter_expression);
+	}
+error:
+	/*
+	 * Return directly to the caller and don't ask the sessiond since something
+	 * went wrong in the parsing of data above.
+	 */
+	return ret;
+
+ask_sessiond:
+	ret = lttng_ctl_ask_sessiond(&lsm, NULL);
+	return ret;
+}
+
+/*
+ *  Disable event(s) of a channel and domain.
+ *  If no event name is specified, all events are disabled.
+ *  If no channel name is specified, the default 'channel0' is used.
+ *  Returns size of returned session payload data or a negative error code.
+ */
+int lttng_disable_event(struct lttng_handle *handle, const char *name,
+		const char *channel_name)
+{
+	struct lttng_event ev;
+
+	memset(&ev, 0, sizeof(ev));
+	ev.loglevel = -1;
+	ev.type = LTTNG_EVENT_ALL;
+	lttng_ctl_copy_string(ev.name, name, sizeof(ev.name));
+	return lttng_disable_event_ext(handle, &ev, channel_name, NULL);
 }
 
 /*
@@ -1128,6 +1276,62 @@ int lttng_disable_channel(struct lttng_handle *handle, const char *name)
 	return lttng_ctl_ask_sessiond(&lsm, NULL);
 }
 
+/*
+ *  Add PID to session tracker.
+ *  Return 0 on success else a negative LTTng error code.
+ */
+int lttng_track_pid(struct lttng_handle *handle, int pid)
+{
+	struct lttcomm_session_msg lsm;
+
+	/*
+	 * NULL arguments are forbidden. No default values.
+	 */
+	if (handle == NULL) {
+		return -LTTNG_ERR_INVALID;
+	}
+
+	memset(&lsm, 0, sizeof(lsm));
+
+	lsm.cmd_type = LTTNG_TRACK_PID;
+	lsm.u.pid_tracker.pid = pid;
+
+	lttng_ctl_copy_lttng_domain(&lsm.domain, &handle->domain);
+
+	lttng_ctl_copy_string(lsm.session.name, handle->session_name,
+			sizeof(lsm.session.name));
+
+	return lttng_ctl_ask_sessiond(&lsm, NULL);
+}
+
+/*
+ *  Remove PID from session tracker.
+ *  Return 0 on success else a negative LTTng error code.
+ */
+int lttng_untrack_pid(struct lttng_handle *handle, int pid)
+{
+	struct lttcomm_session_msg lsm;
+
+	/*
+	 * NULL arguments are forbidden. No default values.
+	 */
+	if (handle == NULL) {
+		return -LTTNG_ERR_INVALID;
+	}
+
+	memset(&lsm, 0, sizeof(lsm));
+
+	lsm.cmd_type = LTTNG_UNTRACK_PID;
+	lsm.u.pid_tracker.pid = pid;
+
+	lttng_ctl_copy_lttng_domain(&lsm.domain, &handle->domain);
+
+	lttng_ctl_copy_string(lsm.session.name, handle->session_name,
+			sizeof(lsm.session.name));
+
+	return lttng_ctl_ask_sessiond(&lsm, NULL);
+}
+
 /*
  *  Lists all available tracepoints of domain.
  *  Sets the contents of the events array.
@@ -1184,6 +1388,35 @@ int lttng_list_tracepoint_fields(struct lttng_handle *handle,
 	return ret / sizeof(struct lttng_event_field);
 }
 
+/*
+ *  Lists all available kernel system calls. Allocates and sets the contents of
+ *  the events array.
+ *
+ *  Returns the number of lttng_event entries in events; on error, returns a
+ *  negative value.
+ */
+int lttng_list_syscalls(struct lttng_event **events)
+{
+	int ret;
+	struct lttcomm_session_msg lsm;
+
+	if (!events) {
+		return -LTTNG_ERR_INVALID;
+	}
+
+	memset(&lsm, 0, sizeof(lsm));
+	lsm.cmd_type = LTTNG_LIST_SYSCALLS;
+	/* Force kernel domain for system calls. */
+	lsm.domain.type = LTTNG_DOMAIN_KERNEL;
+
+	ret = lttng_ctl_ask_sessiond(&lsm, (void **) events);
+	if (ret < 0) {
+		return ret;
+	}
+
+	return ret / sizeof(struct lttng_event);
+}
+
 /*
  *  Returns a human readable string describing
  *  the error code (a negative value).
@@ -1271,6 +1504,26 @@ int lttng_list_sessions(struct lttng_session **sessions)
 	return ret / sizeof(struct lttng_session);
 }
 
+int lttng_set_session_shm_path(const char *session_name,
+		const char *shm_path)
+{
+	struct lttcomm_session_msg lsm;
+
+	if (session_name == NULL) {
+		return -LTTNG_ERR_INVALID;
+	}
+
+	memset(&lsm, 0, sizeof(lsm));
+	lsm.cmd_type = LTTNG_SET_SESSION_SHM_PATH;
+
+	lttng_ctl_copy_string(lsm.session.name, session_name,
+			sizeof(lsm.session.name));
+	lttng_ctl_copy_string(lsm.u.set_shm_path.shm_path, shm_path,
+			sizeof(lsm.u.set_shm_path.shm_path));
+
+	return lttng_ctl_ask_sessiond(&lsm, NULL);
+}
+
 /*
  *  Ask the session daemon for all available domains of a session.
  *  Sets the contents of the domains array.
@@ -1697,7 +1950,7 @@ int lttng_create_session_live(const char *name, const char *url,
 	struct lttcomm_session_msg lsm;
 	struct lttng_uri *uris = NULL;
 
-	if (name == NULL) {
+	if (name == NULL || timer_interval == 0) {
 		return -LTTNG_ERR_INVALID;
 	}
 
@@ -1733,6 +1986,52 @@ end:
 	return ret;
 }
 
+/*
+ * List PIDs in the tracker.
+ *
+ * @enabled is set to whether the PID tracker is enabled.
+ * @pids is set to an allocated array of PIDs currently tracked. On
+ * success, @pids must be freed by the caller.
+ * @nr_pids is set to the number of entries contained by the @pids array.
+ *
+ * Returns 0 on success, else a negative LTTng error code.
+ */
+int lttng_list_tracker_pids(struct lttng_handle *handle,
+		int *_enabled, int32_t **_pids, size_t *_nr_pids)
+{
+	int ret;
+	int enabled = 1;
+	struct lttcomm_session_msg lsm;
+	size_t nr_pids;
+	int32_t *pids;
+
+	if (handle == NULL) {
+		return -LTTNG_ERR_INVALID;
+	}
+
+	memset(&lsm, 0, sizeof(lsm));
+	lsm.cmd_type = LTTNG_LIST_TRACKER_PIDS;
+	lttng_ctl_copy_string(lsm.session.name, handle->session_name,
+			sizeof(lsm.session.name));
+	lttng_ctl_copy_lttng_domain(&lsm.domain, &handle->domain);
+
+	ret = lttng_ctl_ask_sessiond(&lsm, (void **) &pids);
+	if (ret < 0) {
+		return ret;
+	}
+	nr_pids = ret / sizeof(int32_t);
+	if (nr_pids == 1 && pids[0] == -1) {
+		free(pids);
+		pids = NULL;
+		enabled = 0;
+		nr_pids = 0;
+	}
+	*_enabled = enabled;
+	*_pids = pids;
+	*_nr_pids = nr_pids;
+	return 0;
+}
+
 /*
  * lib constructor
  */
