@@ -39,11 +39,12 @@
 #define RANDOM_STRING_LEN	11
 
 /* Number of TAP tests in this file */
-#define NUM_TESTS 10
+#define NUM_TESTS 11
 
 /* For error.h */
 int lttng_opt_quiet = 1;
 int lttng_opt_verbose;
+int lttng_opt_mi;
 
 int ust_consumerd32_fd;
 int ust_consumerd64_fd;
@@ -100,7 +101,7 @@ static void test_create_ust_channel(void)
 
 	strncpy(attr.name, "channel0", 8);
 
-	uchan = trace_ust_create_channel(&attr);
+	uchan = trace_ust_create_channel(&attr, LTTNG_DOMAIN_UST);
 	ok(uchan != NULL, "Create UST channel");
 
 	ok(uchan->enabled == 0 &&
@@ -124,7 +125,7 @@ static void test_create_ust_event(void)
 	ev.type = LTTNG_EVENT_TRACEPOINT;
 	ev.loglevel_type = LTTNG_EVENT_LOGLEVEL_ALL;
 
-	event = trace_ust_create_event(&ev, NULL, NULL, NULL);
+	event = trace_ust_create_event(&ev, NULL, NULL, NULL, false);
 
 	ok(event != NULL, "Create UST event");
 
@@ -156,10 +157,16 @@ static void test_create_ust_event_exclusion(void)
 
 	/* set up an exclusion set */
 	exclusion = zmalloc(sizeof(*exclusion) + LTTNG_SYMBOL_NAME_LEN);
+	if (!exclusion) {
+		PERROR("zmalloc");
+	}
+
+	ok(exclusion != NULL, "Create UST exclusion");
+
 	exclusion->count = 1;
 	strncpy((char *)(exclusion->names), get_random_string(), LTTNG_SYMBOL_NAME_LEN);
 
-	event = trace_ust_create_event(&ev, NULL, NULL, exclusion);
+	event = trace_ust_create_event(&ev, NULL, NULL, exclusion, false);
 
 	ok(event != NULL, "Create UST event with exclusion");
 
